@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -52,13 +52,15 @@ import net.sourceforge.plantuml.command.regex.MyPattern;
 import net.sourceforge.plantuml.command.regex.Pattern2;
 import net.sourceforge.plantuml.cucadiagram.dot.GraphvizUtils;
 import net.sourceforge.plantuml.preproc.Defines;
+import net.sourceforge.plantuml.security.SFile;
 import net.sourceforge.plantuml.stats.StatsUtils;
 
 public class Option {
 
-	private final List<String> excludes = new ArrayList<String>();
-	private final List<String> config = new ArrayList<String>();
+	private final List<String> excludes = new ArrayList<>();
+	private final List<String> config = new ArrayList<>();
 	private final Map<String, String> defines = new LinkedHashMap<String, String>();
+
 	private String charset;
 	private boolean computeurl = false;
 	private boolean decodeurl = false;
@@ -71,22 +73,27 @@ public class Option {
 	private OptionPreprocOutputMode preprocessorOutput = null;
 	private boolean failfast = false;
 	private boolean failfast2 = false;
-	private boolean pattern = false;
+	private boolean noerror = false;
+
 	private boolean duration = false;
 	private boolean debugsvek = false;
 	private boolean splash = false;
 	private boolean textProgressBar = false;
 	private int nbThreads = 0;
 	private int ftpPort = -1;
+	private String picowebBindAddress = null;
+	private int picowebPort = -1;
 	private boolean hideMetadata = false;
 	private boolean checkMetadata = false;
+	private int stdrpt = 0;
 	private int imageIndex = 0;
+	private String fileDir;
 
 	private File outputDir = null;
 	private File outputFile = null;
 	private String filename;
 
-	private final List<String> result = new ArrayList<String>();
+	private final List<String> result = new ArrayList<>();
 
 	public Option() {
 	}
@@ -107,6 +114,7 @@ public class Option {
 		if (arg.length == 0) {
 			OptionFlags.getInstance().setGui(true);
 		}
+		initInclude(GraphvizUtils.getenvDefaultConfigFilename());
 		for (int i = 0; i < arg.length; i++) {
 			String s = arg[i];
 			if (s.equalsIgnoreCase("-headless")) {
@@ -183,11 +191,15 @@ public class Option {
 					continue;
 				}
 				filename = StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(arg[i]);
+			} else if (s.equalsIgnoreCase("-filedir")) {
+				i++;
+				if (i == arg.length) {
+					continue;
+				}
+				fileDir = StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(arg[i]);
 			} else if (s.startsWith("-o") && s.length() > 3) {
 				s = s.substring(2);
 				outputDir = new File(StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(s));
-			} else if (s.equalsIgnoreCase("-recurse") || s.equalsIgnoreCase("-r")) {
-				// recurse = true;
 			} else if (s.equalsIgnoreCase("-exclude") || s.equalsIgnoreCase("-x")) {
 				i++;
 				if (i == arg.length) {
@@ -218,8 +230,16 @@ public class Option {
 				this.failfast = true;
 			} else if (s.equalsIgnoreCase("-failfast2")) {
 				this.failfast2 = true;
+			} else if (s.equalsIgnoreCase("-noerror")) {
+				this.noerror = true;
 			} else if (s.equalsIgnoreCase("-checkonly")) {
 				this.checkOnly = true;
+			} else if (s.equalsIgnoreCase("-theme")) {
+				i++;
+				if (i == arg.length) {
+					continue;
+				}
+				this.config.add(0, "!theme " + arg[i]);
 			} else if (s.equalsIgnoreCase("-config")) {
 				i++;
 				if (i == arg.length) {
@@ -247,8 +267,6 @@ public class Option {
 				pipeMap = true;
 			} else if (s.equalsIgnoreCase("-pipenostderr")) {
 				pipeNoStdErr = true;
-			} else if (s.equalsIgnoreCase("-pattern")) {
-				pattern = true;
 			} else if (s.equalsIgnoreCase("-syntax")) {
 				syntax = true;
 				OptionFlags.getInstance().setQuiet(true);
@@ -265,8 +283,8 @@ public class Option {
 				if (i == arg.length) {
 					continue;
 				}
-				OptionFlags.getInstance().setLogData(
-						new File(StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(arg[i])));
+				OptionFlags.getInstance()
+						.setLogData(new SFile(StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(arg[i])));
 			} else if (s.equalsIgnoreCase("-word")) {
 				OptionFlags.getInstance().setWord(true);
 				OptionFlags.getInstance().setQuiet(true);
@@ -279,15 +297,21 @@ public class Option {
 				OptionPrint.printVersion();
 			} else if (s.matches("(?i)^-li[sc][ea]n[sc]e\\s*$")) {
 				OptionPrint.printLicense();
-			} else if (s.equalsIgnoreCase("-checkversion")) {
-				OptionPrint.checkVersion();
+			} else if (s.startsWith("-DPLANTUML_LIMIT_SIZE=")) {
+				final String v = s.substring("-DPLANTUML_LIMIT_SIZE=".length());
+				if (v.matches("\\d+")) {
+					System.setProperty("PLANTUML_LIMIT_SIZE", v);
+				}
 			} else if (s.startsWith("-D")) {
 				manageDefine(s.substring(2));
 			} else if (s.startsWith("-S")) {
 				manageSkinParam(s.substring(2));
+			} else if (s.startsWith("-P")) {
+				managePragma(s.substring(2));
 			} else if (s.equalsIgnoreCase("-testdot")) {
 				OptionPrint.printTestDot();
-			} else if (s.equalsIgnoreCase("-about") || s.equalsIgnoreCase("-author") || s.equalsIgnoreCase("-authors")) {
+			} else if (s.equalsIgnoreCase("-about") || s.equalsIgnoreCase("-author")
+					|| s.equalsIgnoreCase("-authors")) {
 				OptionPrint.printAbout();
 			} else if (s.equalsIgnoreCase("-help") || s.equalsIgnoreCase("-h") || s.equalsIgnoreCase("-?")) {
 				OptionPrint.printHelp();
@@ -297,8 +321,6 @@ public class Option {
 				OptionFlags.getInstance().setGui(true);
 			} else if (s.equalsIgnoreCase("-encodesprite")) {
 				OptionFlags.getInstance().setEncodesprite(true);
-				// } else if (s.equalsIgnoreCase("-nosuggestengine")) {
-				// OptionFlags.getInstance().setUseSuggestEngine(false);
 			} else if (s.equalsIgnoreCase("-printfonts")) {
 				OptionFlags.getInstance().setPrintFonts(true);
 			} else if (s.equalsIgnoreCase("-dumphtmlstats")) {
@@ -339,6 +361,12 @@ public class Option {
 				preprocessorOutput = OptionPreprocOutputMode.CYPHER;
 			} else if (s.equalsIgnoreCase("-checkmetadata")) {
 				checkMetadata = true;
+			} else if (s.equalsIgnoreCase("-stdrpt:1")) {
+				stdrpt = 1;
+			} else if (s.equalsIgnoreCase("-stdrpt:2")) {
+				stdrpt = 2;
+			} else if (s.equalsIgnoreCase("-stdrpt")) {
+				stdrpt = 2;
 			} else if (s.equalsIgnoreCase("-pipeimageindex")) {
 				i++;
 				if (i == arg.length) {
@@ -355,6 +383,10 @@ public class Option {
 				} else {
 					this.ftpPort = Integer.parseInt(s.substring(x + 1));
 				}
+			} else if (StringUtils.goLowerCase(s).startsWith("-picoweb")) {
+				final String[] parts = s.split(":");
+				this.picowebPort = parts.length > 1 ? Integer.parseInt(parts[1]) : 8080;
+				this.picowebBindAddress = parts.length > 2 ? parts[2] : null;
 			} else if (s.startsWith("-c")) {
 				s = s.substring(2);
 				config.add(StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(s));
@@ -364,37 +396,67 @@ public class Option {
 		}
 	}
 
+	public Stdrpt getStdrpt() {
+		if (stdrpt == 1) {
+			return new StdrptV1();
+		}
+		if (stdrpt == 2) {
+			return new StdrptV2();
+		}
+		// Legacy case
+		if (isPipe() || isPipeMap() || isSyntax()) {
+			return new StdrptPipe0();
+		}
+		return new StdrptNull();
+	}
+
 	public int getFtpPort() {
 		return ftpPort;
 	}
 
-	private void addInConfig(final FileReader source) throws IOException {
-		BufferedReader br = null;
+	public String getPicowebBindAddress() {
+		return picowebBindAddress;
+	}
+
+	public int getPicowebPort() {
+		return picowebPort;
+	}
+
+	private void addInConfig(BufferedReader br) throws IOException {
+		if (br == null) {
+			return;
+		}
 		try {
-			br = new BufferedReader(source);
 			String s = null;
 			while ((s = br.readLine()) != null) {
 				config.add(s);
 			}
 		} finally {
-			if (br != null) {
-				br.close();
-			}
+			br.close();
 		}
 	}
 
 	public void initConfig(String filename) throws IOException {
-		addInConfig(new FileReader(filename));
+		final BufferedReader br = new BufferedReader(new FileReader(filename));
+		addInConfig(br);
 	}
 
 	private void initInclude(String filename) throws IOException {
+		if (filename == null) {
+			return;
+		}
 		if (filename.contains("*")) {
-			final FileGroup group = new FileGroup(filename, Collections.<String> emptyList(), null);
+			final FileGroup group = new FileGroup(filename, Collections.<String>emptyList(), null);
 			for (File f : group.getFiles()) {
-				addInConfig(new FileReader(f));
+				if (f.exists() && f.canRead()) {
+					addInConfig(new BufferedReader(new FileReader(f)));
+				}
 			}
 		} else {
-			addInConfig(new FileReader(filename));
+			final File f = new File(filename);
+			if (f.exists() && f.canRead()) {
+				addInConfig(new BufferedReader(new FileReader(f)));
+			}
 		}
 	}
 
@@ -406,27 +468,36 @@ public class Option {
 		}
 	}
 
+	private void managePragma(String s) {
+		final Pattern2 p = MyPattern.cmpile("^(\\w+)(?:=(.*))?$");
+		final Matcher2 m = p.matcher(s);
+		if (m.find()) {
+			final String var = m.group(1);
+			final String value = StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(m.group(2));
+			if (var != null && value != null) {
+				config.add("!pragma " + var + " " + value);
+			}
+		}
+	}
+
 	private void manageSkinParam(String s) {
 		final Pattern2 p = MyPattern.cmpile("^(\\w+)(?:=(.*))?$");
 		final Matcher2 m = p.matcher(s);
 		if (m.find()) {
-			skinParam(m.group(1), m.group(2));
+			final String var = m.group(1);
+			final String value = m.group(2);
+			if (var != null && value != null) {
+				config.add("skinparamlocked " + var + " " + value);
+			}
 		}
 	}
 
-	private void skinParam(String var, String value) {
-		if (var != null && value != null) {
-			config.add("skinparamlocked " + var + " " + value);
-		}
-
-	}
-
 	public final File getOutputDir() {
 		return outputDir;
 	}
 
 	public final static String getPattern() {
-		return "(?i)^.*\\.(txt|tex|java|htm|html|c|h|cpp|apt|pu|pump|hpp|hh)$";
+		return "(?i)^.*\\.(txt|tex|java|htm|html|c|h|cpp|apt|pu|puml|hpp|hh)$";
 	}
 
 	public void setOutputDir(File f) {
@@ -437,10 +508,26 @@ public class Option {
 		return Collections.unmodifiableList(excludes);
 	}
 
-	public Defines getDefaultDefines(File f) {
+	public Defines getDefaultDefines(SFile f) {
 		final Defines result = Defines.createWithFileName(f);
 		for (Map.Entry<String, String> ent : defines.entrySet()) {
-			result.define(ent.getKey(), Arrays.asList(ent.getValue()), false);
+			String value = ent.getValue();
+			if (value == null) {
+				value = "";
+			}
+			result.define(ent.getKey(), Arrays.asList(value), false, null);
+		}
+		return result;
+	}
+
+	public Defines getDefaultDefines(java.io.File f) {
+		final Defines result = Defines.createWithFileName(f);
+		for (Map.Entry<String, String> ent : defines.entrySet()) {
+			String value = ent.getValue();
+			if (value == null) {
+				value = "";
+			}
+			result.define(ent.getKey(), Arrays.asList(value), false, null);
 		}
 		return result;
 	}
@@ -449,7 +536,7 @@ public class Option {
 		final Defines result = Defines.createEmpty();
 		result.overrideFilename(filename);
 		for (Map.Entry<String, String> ent : defines.entrySet()) {
-			result.define(ent.getKey(), Arrays.asList(ent.getValue()), false);
+			result.define(ent.getKey(), Arrays.asList(ent.getValue()), false, null);
 		}
 		return result;
 	}
@@ -495,10 +582,6 @@ public class Option {
 		return syntax;
 	}
 
-	public final boolean isPattern() {
-		return pattern;
-	}
-
 	public FileFormatOption getFileFormatOption() {
 		if (debugsvek) {
 			fileFormatOption.setDebugSvek(true);
@@ -549,6 +632,14 @@ public class Option {
 		this.failfast2 = failfast2;
 	}
 
+	public final void setNoerror(boolean noerror) {
+		this.noerror = noerror;
+	}
+
+	public final boolean isNoerror() {
+		return noerror;
+	}
+
 	public final File getOutputFile() {
 		return outputFile;
 	}
@@ -601,4 +692,7 @@ public class Option {
 	// this.preprocessorOutput = preprocessorOutput;
 	// }
 
+	public String getFileDir() {
+		return fileDir;
+	}
 }
