@@ -46,10 +46,8 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.PrintWriter;
 import java.net.URL;
-import java.time.format.TextStyle;
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Locale;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.TimeUnit;
@@ -63,8 +61,8 @@ import net.sourceforge.plantuml.ftp.FtpServer;
 import net.sourceforge.plantuml.picoweb.PicoWebServer;
 import net.sourceforge.plantuml.png.MetadataTag;
 import net.sourceforge.plantuml.preproc.Stdlib;
-import net.sourceforge.plantuml.security.SImageIO;
 import net.sourceforge.plantuml.security.SFile;
+import net.sourceforge.plantuml.security.SImageIO;
 import net.sourceforge.plantuml.security.SecurityUtils;
 import net.sourceforge.plantuml.sprite.SpriteGrayLevel;
 import net.sourceforge.plantuml.sprite.SpriteUtils;
@@ -85,10 +83,10 @@ public class Run {
 		if (argsArray.length > 0 && argsArray[0].equalsIgnoreCase("-headless")) {
 			System.setProperty("java.awt.headless", "true");
 		}
-		if (argsArray.length > 0 && argsArray[0].equalsIgnoreCase("--de")) {
-			debugGantt();
-			return;
-		}
+//		if (argsArray.length > 0 && argsArray[0].equalsIgnoreCase("--de")) {
+//			debugGantt();
+//			return;
+//		}
 		saveCommandLine(argsArray);
 		final Option option = new Option(argsArray);
 		ProgressBar.setEnable(option.isTextProgressBar());
@@ -576,16 +574,16 @@ public class Run {
 		error.goOk();
 	}
 
-	public static void debugGantt() {
-		final Locale locale = Locale.GERMAN;
-		for (java.time.Month month : java.time.Month.values()) {
-			System.err.println("Testing locale " + locale + " " + month);
-			for (TextStyle style : TextStyle.values()) {
-				final String s = month.getDisplayName(style, locale);
-				System.err.println(style + " --> '" + s + "'");
-
-			}
-		}
-	}
+//	public static void debugGantt() {
+//		final Locale locale = Locale.GERMAN;
+//		for (java.time.Month month : java.time.Month.values()) {
+//			System.err.println("Testing locale " + locale + " " + month);
+//			for (TextStyle style : TextStyle.values()) {
+//				final String s = month.getDisplayName(style, locale);
+//				System.err.println(style + " --> '" + s + "'");
+//
+//			}
+//		}
+//	}
 
 }
