@@ -43,6 +43,7 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.EnumSet;
 import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
@@ -147,9 +148,8 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public void muteStyle(Style modifiedStyle) {
-		if (UseStyle.useBetaStyle()) {
+		if (UseStyle.useBetaStyle())
 			styleBuilder = getCurrentStyleBuilder().muteStyle(modifiedStyle);
-		}
 	}
 
 	public String getDefaultSkin() {
@@ -166,7 +166,6 @@ public class SkinParam implements ISkinParam {
 		if (result == null) {
 			result = tmp.loadSkin("plantuml.skin");
 		}
-
 		return result;
 	}
 
@@ -178,6 +177,7 @@ public class SkinParam implements ISkinParam {
 	private static final Pattern2 stereoPattern = MyPattern.cmpile(stereoPatternString);
 
 	private final Map<String, String> params = new HashMap<String, String>();
+	private final Map<String, String> paramsPendingForStyleMigration = new LinkedHashMap<String, String>();
 	private final Map<String, String> svgCharSizes = new HashMap<String, String>();
 	private Rankdir rankdir = Rankdir.TOP_TO_BOTTOM;
 	private final UmlDiagramType type;
@@ -194,15 +194,17 @@ public class SkinParam implements ISkinParam {
 	public void setParam(String key, String value) {
 		for (String key2 : cleanForKey(key)) {
 			params.put(key2, StringUtils.trin(value));
-			if (key2.startsWith("usebetastyle")) {
-				final boolean betastyle = "true".equalsIgnoreCase(value);
-				UseStyle.setBetaStyle(betastyle);
-			}
+			if (key2.startsWith("usebetastyle") && "true".equalsIgnoreCase(value))
+				UseStyle.setBetaStyle(true);
+
 			if (UseStyle.useBetaStyle()) {
+				applyPendingStyleMigration();
 				final FromSkinparamToStyle convertor = new FromSkinparamToStyle(key2, value, getCurrentStyleBuilder());
-				for (Style style : convertor.getStyles()) {
+				for (Style style : convertor.getStyles())
 					muteStyle(style);
-				}
+
+			} else {
+				paramsPendingForStyleMigration.put(key, value);
 			}
 		}
 		if ("style".equalsIgnoreCase(key) && "strictuml".equalsIgnoreCase(value)) {
@@ -221,6 +223,16 @@ public class SkinParam implements ISkinParam {
 		}
 	}
 
+	public void applyPendingStyleMigration() {
+		for (Entry<String, String> ent : paramsPendingForStyleMigration.entrySet()) {
+			final FromSkinparamToStyle convertor = new FromSkinparamToStyle(ent.getKey(), ent.getValue(),
+					getCurrentStyleBuilder());
+			for (Style style : convertor.getStyles())
+				muteStyle(style);
+		}
+		paramsPendingForStyleMigration.clear();
+	}
+
 	public static SkinParam create(UmlDiagramType type) {
 		return new SkinParam(type);
 	}
@@ -289,6 +301,8 @@ public class SkinParam implements ISkinParam {
 	}
 
 	public String getValue(String key) {
+		if (UseStyle.useBetaStyle())
+			applyPendingStyleMigration();
 		for (String key2 : cleanForKey(key)) {
 			final String result = params.get(key2);
 			if (result != null) {
