@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -36,11 +36,13 @@
 package net.sourceforge.plantuml;
 
 import java.io.File;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.util.Collections;
 import java.util.List;
 
 import net.sourceforge.plantuml.preproc.Defines;
+import net.sourceforge.plantuml.security.SFile;
 
 public class SourceFileReader extends SourceFileReaderAbstract implements ISourceFileReader {
 
@@ -49,44 +51,38 @@ public class SourceFileReader extends SourceFileReaderAbstract implements ISourc
 	}
 
 	public SourceFileReader(File file, File outputDirectory, String charset) throws IOException {
-		this(Defines.createWithFileName(file), file, outputDirectory, Collections.<String> emptyList(), charset,
+		this(Defines.createWithFileName(file), file, outputDirectory, Collections.<String>emptyList(), charset,
 				new FileFormatOption(FileFormat.PNG));
 	}
 
 	public SourceFileReader(final File file, File outputDirectory) throws IOException {
-		this(Defines.createWithFileName(file), file, outputDirectory, Collections.<String> emptyList(), null,
+		this(Defines.createWithFileName(file), file, outputDirectory, Collections.<String>emptyList(), null,
 				new FileFormatOption(FileFormat.PNG));
 	}
 
 	public SourceFileReader(final File file, File outputDirectory, FileFormatOption fileFormatOption)
 			throws IOException {
-		this(Defines.createWithFileName(file), file, outputDirectory, Collections.<String> emptyList(), null,
+		this(Defines.createWithFileName(file), file, outputDirectory, Collections.<String>emptyList(), null,
 				fileFormatOption);
 	}
 
-	public SourceFileReader(Defines defines, final File file, File outputDirectory, List<String> config,
-			String charset, FileFormatOption fileFormatOption) throws IOException {
-		this.file = file;
-		this.fileFormatOption = fileFormatOption;
-		if (file.exists() == false) {
-			throw new IllegalArgumentException();
-		}
-		FileSystem.getInstance().setCurrentDir(file.getAbsoluteFile().getParentFile());
+	public SourceFileReader(Defines defines, final File file, File outputDirectory, List<String> config, String charset,
+			FileFormatOption fileFormatOption) throws IOException {
+		super(file, fileFormatOption, defines, config, charset);
+		FileSystem.getInstance().setCurrentDir(SFile.fromFile(file.getAbsoluteFile().getParentFile()));
 		if (outputDirectory == null) {
 			outputDirectory = file.getAbsoluteFile().getParentFile();
 		} else if (outputDirectory.isAbsolute() == false) {
-			outputDirectory = FileSystem.getInstance().getFile(outputDirectory.getPath());
+			outputDirectory = FileSystem.getInstance().getFile(outputDirectory.getPath()).conv();
 		}
 		if (outputDirectory.exists() == false) {
 			outputDirectory.mkdirs();
 		}
 		this.outputDirectory = outputDirectory;
 
-		builder = new BlockUmlBuilder(config, charset, defines, getReader(charset), file.getAbsoluteFile()
-				.getParentFile(), file.getAbsolutePath());
 	}
 
-	private File getDirIfDirectory(String newName) {
+	private File getDirIfDirectory(String newName) throws FileNotFoundException {
 		Log.info("Checking=" + newName);
 		if (endsWithSlashOrAntislash(newName)) {
 			Log.info("It ends with / so it looks like a directory");
@@ -134,7 +130,7 @@ public class SourceFileReader extends SourceFileReaderAbstract implements ISourc
 	}
 
 	@Override
-	protected SuggestedFile getSuggestedFile(BlockUml blockUml) {
+	protected SuggestedFile getSuggestedFile(BlockUml blockUml) throws FileNotFoundException {
 		final String newName = blockUml.getFileOrDirname();
 		SuggestedFile suggested = null;
 		if (newName != null) {
@@ -159,5 +155,4 @@ public class SourceFileReader extends SourceFileReaderAbstract implements ISourc
 		return suggested;
 	}
 
-
 }
