@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -39,6 +39,7 @@ import java.util.List;
 
 import net.sourceforge.plantuml.BackSlash;
 import net.sourceforge.plantuml.Direction;
+import net.sourceforge.plantuml.StringLocated;
 import net.sourceforge.plantuml.StringUtils;
 import net.sourceforge.plantuml.Url;
 import net.sourceforge.plantuml.UrlBuilder;
@@ -49,6 +50,7 @@ import net.sourceforge.plantuml.command.BlocLines;
 import net.sourceforge.plantuml.command.CommandExecutionResult;
 import net.sourceforge.plantuml.command.CommandMultilines2;
 import net.sourceforge.plantuml.command.MultilinesStrategy;
+import net.sourceforge.plantuml.command.regex.IRegex;
 import net.sourceforge.plantuml.command.regex.MyPattern;
 import net.sourceforge.plantuml.command.regex.RegexConcat;
 import net.sourceforge.plantuml.command.regex.RegexLeaf;
@@ -59,12 +61,16 @@ import net.sourceforge.plantuml.cucadiagram.Code;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.cucadiagram.GroupType;
 import net.sourceforge.plantuml.cucadiagram.IEntity;
+import net.sourceforge.plantuml.cucadiagram.Ident;
 import net.sourceforge.plantuml.cucadiagram.LeafType;
 import net.sourceforge.plantuml.cucadiagram.Link;
 import net.sourceforge.plantuml.cucadiagram.LinkDecor;
 import net.sourceforge.plantuml.cucadiagram.LinkType;
+import net.sourceforge.plantuml.cucadiagram.NamespaceStrategy;
 import net.sourceforge.plantuml.cucadiagram.Stereotype;
+import net.sourceforge.plantuml.descdiagram.command.CommandLinkElement;
 import net.sourceforge.plantuml.graphic.color.ColorType;
+import net.sourceforge.plantuml.ugraphic.color.NoSuchColorException;
 
 public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram> {
 
@@ -74,66 +80,72 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 
 	@Override
 	public String getPatternEnd() {
-		return "(?i)^[%s]*([^%g]*)[%g](?:[%s]+as[%s]+([\\p{L}0-9][\\p{L}0-9_.]*))?[%s]*(\\<\\<.*\\>\\>)?[%s]*(?:in[%s]+([%g][^%g]+[%g]|\\S+))?[%s]*(#\\w+)?$";
+		return "^[%s]*([^%g]*)[%g](?:[%s]+as[%s]+([%pLN][%pLN_.]*))?[%s]*(\\<\\<.*\\>\\>)?[%s]*(?:in[%s]+([%g][^%g]+[%g]|\\S+))?[%s]*(#\\w+)?$";
 	}
 
-	static RegexConcat getRegexConcat() {
-		return new RegexConcat(new RegexLeaf("^"), //
+	static IRegex getRegexConcat() {
+		return RegexConcat.build(CommandLinkLongActivity.class.getName(), RegexLeaf.start(), //
 				new RegexOptional(//
 						new RegexOr("FIRST", //
 								new RegexLeaf("STAR", "(\\(\\*(top)?\\))"), //
-								new RegexLeaf("CODE", "([\\p{L}0-9][\\p{L}0-9_.]*)"), //
-								new RegexLeaf("BAR", "(?:==+)[%s]*([\\p{L}0-9_.]+)[%s]*(?:==+)"), //
-								new RegexLeaf("QUOTED", "[%g]([^%g]+)[%g](?:[%s]+as[%s]+([\\p{L}0-9_.]+))?"))), //
-				new RegexLeaf("[%s]*"), //
+								new RegexLeaf("CODE", "([%pLN][%pLN_.]*)"), //
+								new RegexLeaf("BAR", "(?:==+)[%s]*([%pLN_.]+)[%s]*(?:==+)"), //
+								new RegexLeaf("QUOTED", "[%g]([^%g]+)[%g](?:[%s]+as[%s]+([%pLN_.]+))?"))), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("STEREOTYPE", "(\\<\\<.*\\>\\>)?"), //
-				new RegexLeaf("[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("BACKCOLOR", "(#\\w+)?"), //
-				new RegexLeaf("[%s]*"), //
+				RegexLeaf.spaceZeroOrMore(), //
 				new RegexLeaf("URL", "(" + UrlBuilder.getRegexp() + ")?"), //
 
 				new RegexLeaf("ARROW_BODY1", "([-.]+)"), //
-				new RegexLeaf("ARROW_STYLE1",
-						"(?:\\[((?:#\\w+|dotted|dashed|plain|bold|hidden)(?:,#\\w+|,dotted|,dashed|,plain|,bold|,hidden)*)\\])?"), //
+				new RegexLeaf("ARROW_STYLE1", "(?:\\[(" + CommandLinkElement.LINE_STYLE + ")\\])?"), //
 				new RegexLeaf("ARROW_DIRECTION", "(\\*|left|right|up|down|le?|ri?|up?|do?)?"), //
-				new RegexLeaf("ARROW_STYLE2",
-						"(?:\\[((?:#\\w+|dotted|dashed|plain|bold|hidden)(?:,#\\w+|,dotted|,dashed|,plain|,bold|,hidden)*)\\])?"), //
-				new RegexLeaf("ARROW_BODY2", "([-.]*)\\>"), //
+				new RegexLeaf("ARROW_STYLE2", "(?:\\[(" + CommandLinkElement.LINE_STYLE + ")\\])?"), //
+				new RegexLeaf("ARROW_BODY2", "([-.]*)"), //
+				new RegexLeaf("\\>"), //
 
-				new RegexLeaf("[%s]*"), //
-				new RegexLeaf("BRACKET", "(?:\\[([^\\]*]+[^\\]]*)\\])?"), //
-				new RegexLeaf("[%s]*"), //
-				new RegexLeaf("DESC", "[%g]([^%g]*?)"), //
-				new RegexLeaf("[%s]*"), //
-				new RegexLeaf("$"));
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexOptional(new RegexLeaf("BRACKET", "\\[([^\\]*]+[^\\]]*)\\]")), //
+				RegexLeaf.spaceZeroOrMore(), //
+				new RegexLeaf("[%g]"), //
+				new RegexLeaf("DESC", "([^%g]*?)"), //
+				RegexLeaf.spaceZeroOrMore(), //
+				RegexLeaf.end());
 	}
 
-	public CommandExecutionResult executeNow(final ActivityDiagram diagram, BlocLines lines) {
-		lines = lines.trim(false);
-		final RegexResult line0 = getStartingPattern().matcher(StringUtils.trin(lines.getFirst499()));
+	@Override
+	protected CommandExecutionResult executeNow(final ActivityDiagram diagram, BlocLines lines)
+			throws NoSuchColorException {
+		lines = lines.trim();
+		final RegexResult line0 = getStartingPattern().matcher(lines.getFirst().getTrimmed().getString());
 
 		final IEntity entity1 = CommandLinkActivity.getEntity(diagram, line0, true);
+		if (entity1 == null) {
+			return CommandExecutionResult.error("No such entity");
+		}
 
 		if (line0.get("STEREOTYPE", 0) != null) {
-			entity1.setStereotype(new Stereotype(line0.get("STEREOTYPE", 0)));
+			entity1.setStereotype(Stereotype.build(line0.get("STEREOTYPE", 0)));
 		}
-		if (line0.get("BACKCOLOR", 0) != null) {
+		final String stringColor = line0.get("BACKCOLOR", 0);
+		if (stringColor != null) {
 			entity1.setSpecificColorTOBEREMOVED(ColorType.BACK, diagram.getSkinParam().getIHtmlColorSet()
-					.getColorIfValid(line0.get("BACKCOLOR", 0)));
+					.getColor(diagram.getSkinParam().getThemeStyle(), stringColor));
 		}
 		final StringBuilder sb = new StringBuilder();
 
 		final String desc0 = line0.get("DESC", 0);
 		Url urlActivity = null;
 		if (StringUtils.isNotEmpty(desc0)) {
-			urlActivity = extractUrl(diagram, desc0);
+			urlActivity = extractUrlString(diagram, desc0);
 			if (urlActivity == null) {
 				sb.append(desc0);
 				sb.append(BackSlash.BS_BS_N);
 			}
 		}
 		int i = 0;
-		for (CharSequence cs : lines.subExtract(1, 1)) {
+		for (StringLocated cs : lines.subExtract(1, 1)) {
 			i++;
 			if (i == 1 && urlActivity == null) {
 				urlActivity = extractUrl(diagram, cs);
@@ -141,14 +153,14 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 					continue;
 				}
 			}
-			sb.append(cs);
+			sb.append(cs.getString());
 			if (i < lines.size() - 2) {
 				sb.append(BackSlash.BS_BS_N);
 			}
 		}
 
-		final List<String> lineLast = StringUtils.getSplit(MyPattern.cmpile(getPatternEnd()), lines.getLast499()
-				.toString());
+		final List<String> lineLast = StringUtils.getSplit(MyPattern.cmpile(getPatternEnd()),
+				lines.getLast().getString());
 		if (StringUtils.isNotEmpty(lineLast.get(0))) {
 			if (sb.length() > 0 && sb.toString().endsWith(BackSlash.BS_BS_N) == false) {
 				sb.append(BackSlash.BS_BS_N);
@@ -157,7 +169,7 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 		}
 
 		final String display = sb.toString();
-		final Code code = Code.of(lineLast.get(1) == null ? display : lineLast.get(1));
+		final String idShort = lineLast.get(1) == null ? display : lineLast.get(1);
 
 		String partition = null;
 		if (lineLast.get(3) != null) {
@@ -165,9 +177,17 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 			partition = StringUtils.eventuallyRemoveStartingAndEndingDoubleQuote(partition);
 		}
 		if (partition != null) {
-			diagram.getOrCreateGroup(Code.of(partition), Display.getWithNewlines(partition), GroupType.PACKAGE, null);
+			final Ident idNewLong = diagram.buildLeafIdent(partition);
+			diagram.gotoGroup(idNewLong, diagram.buildCode(partition), Display.getWithNewlines(partition),
+					GroupType.PACKAGE, null, NamespaceStrategy.SINGLE);
 		}
-		final IEntity entity2 = diagram.createLeaf(code, Display.getWithNewlines(display), LeafType.ACTIVITY, null);
+		final Ident ident = diagram.buildLeafIdent(idShort);
+		final Code code = diagram.V1972() ? ident : diagram.buildCode(idShort);
+		final IEntity entity2 = diagram.getOrCreate(ident, code, Display.getWithNewlines(display), LeafType.ACTIVITY);
+		if (entity2 == null) {
+			return CommandExecutionResult.error("No such entity");
+		}
+
 		if (partition != null) {
 			diagram.endGroup();
 		}
@@ -176,14 +196,12 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 		}
 
 		if (lineLast.get(2) != null) {
-			entity2.setStereotype(new Stereotype(lineLast.get(2)));
+			entity2.setStereotype(Stereotype.build(lineLast.get(2)));
 		}
 		if (lineLast.get(4) != null) {
-			entity2.setSpecificColorTOBEREMOVED(ColorType.BACK, diagram.getSkinParam().getIHtmlColorSet().getColorIfValid(lineLast.get(4)));
-		}
-
-		if (entity1 == null || entity2 == null) {
-			return CommandExecutionResult.error("No such entity");
+			String s = lineLast.get(4);
+			entity2.setSpecificColorTOBEREMOVED(ColorType.BACK,
+					diagram.getSkinParam().getIHtmlColorSet().getColor(diagram.getSkinParam().getThemeStyle(), s));
 		}
 
 		final String arrowBody1 = CommandLinkClass.notNull(line0.get("ARROW_BODY1", 0));
@@ -198,9 +216,10 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 
 		LinkType type = new LinkType(LinkDecor.ARROW, LinkDecor.NONE);
 		if (arrow.contains(".")) {
-			type = type.getDotted();
+			type = type.goDotted();
 		}
-		Link link = new Link(entity1, entity2, type, linkLabel, lenght);
+		Link link = new Link(entity1, entity2, type, linkLabel, lenght,
+				diagram.getSkinParam().getCurrentStyleBuilder());
 		final Direction direction = StringUtils.getArrowDirection(arrowBody1 + arrowDirection + arrowBody2 + ">");
 		if (direction == Direction.LEFT || direction == Direction.UP) {
 			link = link.getInv();
@@ -212,15 +231,19 @@ public class CommandLinkLongActivity extends CommandMultilines2<ActivityDiagram>
 			link.setUrl(urlLink);
 		}
 
-		CommandLinkClass.applyStyle(line0.getLazzy("ARROW_STYLE", 0), link);
+		link.applyStyle(diagram.getSkinParam().getThemeStyle(), line0.getLazzy("ARROW_STYLE", 0));
 		diagram.addLink(link);
 
 		return CommandExecutionResult.ok();
 	}
 
-	public Url extractUrl(final ActivityDiagram diagram, CharSequence string) {
+	public Url extractUrl(final ActivityDiagram diagram, StringLocated string) {
+		return extractUrlString(diagram, string.getString());
+	}
+
+	public Url extractUrlString(final ActivityDiagram diagram, String string) {
 		final UrlBuilder urlBuilder = new UrlBuilder(diagram.getSkinParam().getValue("topurl"), ModeUrl.STRICT);
-		return urlBuilder.getUrl(string.toString());
+		return urlBuilder.getUrl(string);
 	}
 
 }
