@@ -198,6 +198,8 @@ public class Branch {
 	}
 
 	public ISkinParam skinParam() {
+		if (gtile != null)
+			return gtile.skinParam();
 		return ftile.skinParam();
 	}
 
@@ -226,9 +228,11 @@ public class Branch {
 	}
 
 	public final Display getDisplayPositive() {
-		final LinkRendering in = ftile.getInLinkRendering();
-		if (in != null && Display.isNull(in.getDisplay()) == false) {
-			return in.getDisplay();
+		if (ftile != null) {
+			final LinkRendering in = ftile.getInLinkRendering();
+			if (in != null && Display.isNull(in.getDisplay()) == false) {
+				return in.getDisplay();
+			}
 		}
 		return labelPositive.getDisplay();
 	}
