@@ -37,10 +37,15 @@ package net.sourceforge.plantuml.activitydiagram3;
 
 import java.util.Objects;
 
+import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Ftile;
 import net.sourceforge.plantuml.activitydiagram3.ftile.FtileFactory;
 import net.sourceforge.plantuml.activitydiagram3.ftile.Swimlane;
+import net.sourceforge.plantuml.activitydiagram3.gtile.Gtile;
+import net.sourceforge.plantuml.activitydiagram3.gtile.GtileCircleStart;
 import net.sourceforge.plantuml.command.CommandExecutionResult;
+import net.sourceforge.plantuml.graphic.StringBounder;
+import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
 
 public class InstructionEnd extends MonoSwimable implements Instruction {
 
@@ -51,24 +56,34 @@ public class InstructionEnd extends MonoSwimable implements Instruction {
 		this.inlinkRendering = Objects.requireNonNull(inlinkRendering);
 	}
 
+	@Override
+	public Gtile createGtile(ISkinParam skinParam, StringBounder stringBounder) {
+		return new GtileCircleStart(stringBounder, skinParam, HColorUtils.BLACK, getSwimlaneIn(), null);
+	}
+
+	@Override
 	public Ftile createFtile(FtileFactory factory) {
 		Ftile result = factory.end(getSwimlaneIn());
 		result = eventuallyAddNote(factory, result, result.getSwimlaneIn());
 		return result;
 	}
 
+	@Override
 	public CommandExecutionResult add(Instruction other) {
 		throw new UnsupportedOperationException();
 	}
 
+	@Override
 	final public boolean kill() {
 		return false;
 	}
 
+	@Override
 	public LinkRendering getInLinkRendering() {
 		return inlinkRendering;
 	}
 
+	@Override
 	public boolean containsBreak() {
 		return false;
 	}
