@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -43,6 +43,7 @@ import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.FontParam;
 import net.sourceforge.plantuml.ISkinParam;
 import net.sourceforge.plantuml.Url;
+import net.sourceforge.plantuml.UseStyle;
 import net.sourceforge.plantuml.activitydiagram3.Branch;
 import net.sourceforge.plantuml.activitydiagram3.ForkStyle;
 import net.sourceforge.plantuml.activitydiagram3.Instruction;
@@ -61,11 +62,18 @@ import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileCircleStop;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDecorateIn;
 import net.sourceforge.plantuml.activitydiagram3.ftile.vertical.FtileDecorateOut;
 import net.sourceforge.plantuml.cucadiagram.Display;
-import net.sourceforge.plantuml.graphic.HtmlColor;
+import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.graphic.StringBounder;
+import net.sourceforge.plantuml.graphic.USymbol;
 import net.sourceforge.plantuml.graphic.color.Colors;
 import net.sourceforge.plantuml.skin.rose.Rose;
+import net.sourceforge.plantuml.style.PName;
+import net.sourceforge.plantuml.style.SName;
+import net.sourceforge.plantuml.style.Style;
+import net.sourceforge.plantuml.style.StyleBuilder;
+import net.sourceforge.plantuml.style.StyleSignature;
 import net.sourceforge.plantuml.ugraphic.UFont;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public class VCompactFactory implements FtileFactory {
 
@@ -77,38 +85,74 @@ public class VCompactFactory implements FtileFactory {
 		return stringBounder;
 	}
 
+	public StyleBuilder getCurrentStyleBuilder() {
+		return skinParam.getCurrentStyleBuilder();
+	}
+
 	public VCompactFactory(ISkinParam skinParam, StringBounder stringBounder) {
 		this.skinParam = skinParam;
 		this.stringBounder = stringBounder;
 	}
 
+	final public StyleSignature getDefaultStyleDefinitionCircle() {
+		return StyleSignature.of(SName.root, SName.element, SName.activityDiagram, SName.circle);
+	}
+
 	public Ftile start(Swimlane swimlane) {
-		final HtmlColor color = rose.getHtmlColor(skinParam, ColorParam.activityStart);
-		return new FtileCircleStart(skinParam(), color, swimlane);
+		final HColor color;
+		Style style = null;
+		if (UseStyle.useBetaStyle()) {
+			style = getDefaultStyleDefinitionCircle().getMergedStyle(skinParam.getCurrentStyleBuilder());
+			color = style.value(PName.LineColor).asColor(skinParam.getThemeStyle(), skinParam.getIHtmlColorSet());
+		} else {
+			color = rose.getHtmlColor(skinParam, ColorParam.activityStart);
+		}
+		return new FtileCircleStart(skinParam(), color, swimlane, style);
 	}
 
 	public Ftile stop(Swimlane swimlane) {
-		final HtmlColor color = rose.getHtmlColor(skinParam, ColorParam.activityEnd);
-		return new FtileCircleStop(skinParam(), color, swimlane);
+		final HColor borderColor;
+		Style style = null;
+		final HColor backgroundColor;
+		if (UseStyle.useBetaStyle()) {
+			style = getDefaultStyleDefinitionCircle().getMergedStyle(skinParam.getCurrentStyleBuilder());
+			borderColor = style.value(PName.LineColor).asColor(skinParam.getThemeStyle(), skinParam.getIHtmlColorSet());
+			// backgroundColor =
+			// style.value(PName.BackGroundColor).asColor(skinParam.getIHtmlColorSet());
+			backgroundColor = skinParam.getBackgroundColor();
+		} else {
+			borderColor = rose.getHtmlColor(skinParam, ColorParam.activityEnd);
+			backgroundColor = skinParam.getBackgroundColor();
+		}
+		return new FtileCircleStop(skinParam(), backgroundColor, borderColor, swimlane, style);
 	}
 
-	public Ftile spot(Swimlane swimlane, String spot) {
-		// final HtmlColor color = rose.getHtmlColor(skinParam, ColorParam.activityBackground);
+	public Ftile spot(Swimlane swimlane, String spot, HColor color) {
+		// final HtmlColor color = rose.getHtmlColor(skinParam,
+		// ColorParam.activityBackground);
 		final UFont font = skinParam.getFont(null, false, FontParam.ACTIVITY);
-		return new FtileCircleSpot(skinParam(), swimlane, spot, font);
+		return new FtileCircleSpot(skinParam(), swimlane, spot, font, color);
 	}
 
 	public Ftile end(Swimlane swimlane) {
-		final HtmlColor color = rose.getHtmlColor(skinParam, ColorParam.activityEnd);
-		return new FtileCircleEnd(skinParam(), color, swimlane);
+		final HColor borderColor;
+		Style style = null;
+		final HColor backgroundColor;
+		if (UseStyle.useBetaStyle()) {
+			style = getDefaultStyleDefinitionCircle().getMergedStyle(skinParam.getCurrentStyleBuilder());
+			borderColor = style.value(PName.LineColor).asColor(skinParam.getThemeStyle(), skinParam.getIHtmlColorSet());
+			// backgroundColor =
+			// style.value(PName.BackGroundColor).asColor(skinParam.getIHtmlColorSet());
+			backgroundColor = skinParam.getBackgroundColor();
+		} else {
+			borderColor = rose.getHtmlColor(skinParam, ColorParam.activityEnd);
+			backgroundColor = skinParam.getBackgroundColor();
+		}
+		return new FtileCircleEnd(skinParam(), backgroundColor, borderColor, swimlane, style);
 	}
 
-	public Ftile activity(Display label, Swimlane swimlane, BoxStyle style, Colors colors) {
-		// final HtmlColor borderColor = rose.getHtmlColor(skinParam, ColorParam.activityBorder);
-		// final HtmlColor backColor = color == null ? rose.getHtmlColor(skinParam, ColorParam.activityBackground) :
-		// color;
-		final UFont font = skinParam.getFont(null, false, FontParam.ACTIVITY);
-		return new FtileBox(colors.mute(skinParam), label, font, swimlane, style);
+	public Ftile activity(Display label, Swimlane swimlane, BoxStyle boxStyle, Colors colors, Stereotype stereotype) {
+		return FtileBox.create(colors.mute(skinParam), label, swimlane, boxStyle, stereotype);
 	}
 
 	public Ftile addNote(Ftile ftile, Swimlane swimlane, Collection<PositionedNote> notes) {
@@ -123,20 +167,21 @@ public class VCompactFactory implements FtileFactory {
 		return new FtileAssemblySimple(tile1, tile2);
 	}
 
-	public Ftile repeat(Swimlane swimlane, Swimlane swimlaneOut, Display startLabel, Ftile repeat, Display test,
-			Display yes, Display out, HtmlColor color, LinkRendering backRepeatLinkRendering, Ftile backward,
-			boolean noOut) {
+	public Ftile repeat(BoxStyle boxStyleIn, Swimlane swimlane, Swimlane swimlaneOut, Display startLabel, Ftile repeat,
+			Display test, Display yes, Display out, Colors colors, Ftile backward, boolean noOut,
+			LinkRendering incoming1, LinkRendering incoming2) {
 		return repeat;
 	}
 
-	public Ftile createWhile(Swimlane swimlane, Ftile whileBlock, Display test, Display yes, Display out,
-			LinkRendering afterEndwhile, HtmlColor color, Instruction specialOut) {
+	public Ftile createWhile(LinkRendering afterEndwhile, Swimlane swimlane, Ftile whileBlock, Display test,
+			Display yes, HColor color, Instruction specialOut, Ftile back, LinkRendering incoming1,
+			LinkRendering incoming2) {
 		return whileBlock;
 	}
 
 	public Ftile createIf(Swimlane swimlane, List<Branch> thens, Branch elseBranch, LinkRendering afterEndwhile,
-			LinkRendering topInlinkRendering) {
-		final List<Ftile> ftiles = new ArrayList<Ftile>();
+			LinkRendering topInlinkRendering, Url url) {
+		final List<Ftile> ftiles = new ArrayList<>();
 		for (Branch branch : thens) {
 			ftiles.add(branch.getFtile());
 		}
@@ -144,12 +189,21 @@ public class VCompactFactory implements FtileFactory {
 		return new FtileForkInner(ftiles);
 	}
 
-	public Ftile createParallel(Swimlane swimlane, List<Ftile> all, ForkStyle style, String label) {
+	public Ftile createSwitch(Swimlane swimlane, List<Branch> branches, LinkRendering afterEndwhile,
+			LinkRendering topInlinkRendering, Display labelTest) {
+		final List<Ftile> ftiles = new ArrayList<>();
+		for (Branch branch : branches) {
+			ftiles.add(branch.getFtile());
+		}
+		return new FtileForkInner(ftiles);
+	}
+
+	public Ftile createParallel(List<Ftile> all, ForkStyle style, String label, Swimlane in, Swimlane out) {
 		return new FtileForkInner(all);
 	}
 
-	public Ftile createGroup(Ftile list, Display name, HtmlColor backColor, HtmlColor titleColor, PositionedNote note,
-			HtmlColor borderColor) {
+	public Ftile createGroup(Ftile list, Display name, HColor backColor, HColor titleColor, PositionedNote note,
+			HColor borderColor, USymbol type, double roundCorner) {
 		return list;
 	}
 
