@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -38,7 +38,9 @@ package net.sourceforge.plantuml.descdiagram;
 import java.util.ArrayList;
 import java.util.List;
 
+import net.sourceforge.plantuml.ISkinSimple;
 import net.sourceforge.plantuml.classdiagram.command.CommandHideShow2;
+import net.sourceforge.plantuml.classdiagram.command.CommandNamespaceSeparator;
 import net.sourceforge.plantuml.classdiagram.command.CommandRemoveRestore;
 import net.sourceforge.plantuml.classdiagram.command.CommandUrl;
 import net.sourceforge.plantuml.command.Command;
@@ -46,56 +48,60 @@ import net.sourceforge.plantuml.command.CommandEndPackage;
 import net.sourceforge.plantuml.command.CommandFootboxIgnored;
 import net.sourceforge.plantuml.command.CommandPage;
 import net.sourceforge.plantuml.command.CommandRankDir;
-import net.sourceforge.plantuml.command.UmlDiagramFactory;
-import net.sourceforge.plantuml.command.note.FactoryNoteCommand;
-import net.sourceforge.plantuml.command.note.FactoryNoteOnEntityCommand;
-import net.sourceforge.plantuml.command.note.FactoryNoteOnLinkCommand;
+import net.sourceforge.plantuml.command.PSystemCommandFactory;
+import net.sourceforge.plantuml.command.note.CommandFactoryNote;
+import net.sourceforge.plantuml.command.note.CommandFactoryNoteOnEntity;
+import net.sourceforge.plantuml.command.note.CommandFactoryNoteOnLink;
 import net.sourceforge.plantuml.command.regex.RegexLeaf;
 import net.sourceforge.plantuml.command.regex.RegexOr;
+import net.sourceforge.plantuml.core.UmlSource;
 import net.sourceforge.plantuml.descdiagram.command.CommandArchimate;
+import net.sourceforge.plantuml.descdiagram.command.CommandArchimateMultilines;
 import net.sourceforge.plantuml.descdiagram.command.CommandCreateElementFull;
 import net.sourceforge.plantuml.descdiagram.command.CommandCreateElementMultilines;
 import net.sourceforge.plantuml.descdiagram.command.CommandLinkElement;
-import net.sourceforge.plantuml.descdiagram.command.CommandNamespaceSeparator;
 import net.sourceforge.plantuml.descdiagram.command.CommandNewpage;
 import net.sourceforge.plantuml.descdiagram.command.CommandPackageWithUSymbol;
 
-public class DescriptionDiagramFactory extends UmlDiagramFactory {
+public class DescriptionDiagramFactory extends PSystemCommandFactory {
 
 	@Override
-	public DescriptionDiagram createEmptyDiagram() {
-		return new DescriptionDiagram();
+	public DescriptionDiagram createEmptyDiagram(UmlSource source, ISkinSimple skinParam) {
+		return new DescriptionDiagram(source, skinParam);
 	}
 
 	@Override
 	protected List<Command> createCommands() {
-		final List<Command> cmds = new ArrayList<Command>();
+		final List<Command> cmds = new ArrayList<>();
 
 		cmds.add(new CommandFootboxIgnored());
 		cmds.add(new CommandNamespaceSeparator());
 		cmds.add(new CommandRankDir());
 		cmds.add(new CommandNewpage(this));
-		addCommonCommands(cmds);
+		addCommonCommands1(cmds);
 
-		cmds.add(new CommandPage());
 		cmds.add(new CommandLinkElement());
 		cmds.add(new CommandHideShow2());
 		cmds.add(new CommandRemoveRestore());
 		//
 		cmds.add(new CommandPackageWithUSymbol());
 		cmds.add(new CommandEndPackage());
-		final FactoryNoteCommand factoryNoteCommand = new FactoryNoteCommand();
+		final CommandFactoryNote factoryNoteCommand = new CommandFactoryNote();
 		cmds.add(factoryNoteCommand.createMultiLine(false));
 
-		final FactoryNoteOnEntityCommand factoryNoteOnEntityCommand = new FactoryNoteOnEntityCommand(new RegexOr(
-				"ENTITY", //
-				new RegexLeaf("[\\p{L}0-9_.]+"), //
-				new RegexLeaf("\\(\\)[%s]*[\\p{L}0-9_.]+"), //
-				new RegexLeaf("\\(\\)[%s]*[%g][^%g]+[%g]"), //
-				new RegexLeaf("\\[[^\\]*]+[^\\]]*\\]"), //
-				new RegexLeaf("\\((?!\\*\\))[^\\)]+\\)"), //
-				new RegexLeaf(":[^:]+:"), //
-				new RegexLeaf("[%g][^%g]+[%g]") //
+		final CommandFactoryNoteOnLink factoryNoteOnLinkCommand = new CommandFactoryNoteOnLink();
+		cmds.add(factoryNoteOnLinkCommand.createSingleLine());
+		cmds.add(factoryNoteOnLinkCommand.createMultiLine(false));
+		
+		final CommandFactoryNoteOnEntity factoryNoteOnEntityCommand = new CommandFactoryNoteOnEntity("desc",
+				new RegexOr("ENTITY", //
+						new RegexLeaf("[%pLN_.]+"), //
+						new RegexLeaf("\\(\\)[%s]*[%pLN_.]+"), //
+						new RegexLeaf("\\(\\)[%s]*[%g][^%g]+[%g]"), //
+						new RegexLeaf("\\[[^\\]*]+[^\\]]*\\]"), //
+						new RegexLeaf("\\((?!\\*\\))[^\\)]+\\)"), //
+						new RegexLeaf(":[^:]+:"), //
+						new RegexLeaf("[%g][^%g]+[%g]") //
 				));
 		cmds.add(factoryNoteOnEntityCommand.createSingleLine());
 
@@ -109,13 +115,11 @@ public class DescriptionDiagramFactory extends UmlDiagramFactory {
 		cmds.add(factoryNoteOnEntityCommand.createMultiLine(false));
 		cmds.add(factoryNoteCommand.createMultiLine(false));
 
-		final FactoryNoteOnLinkCommand factoryNoteOnLinkCommand = new FactoryNoteOnLinkCommand();
-		cmds.add(factoryNoteOnLinkCommand.createSingleLine());
-		cmds.add(factoryNoteOnLinkCommand.createMultiLine(false));
 
 		// cmds.add(new CommandHideShowSpecificClass());
 
 		cmds.add(new CommandArchimate());
+		cmds.add(new CommandArchimateMultilines());
 		cmds.add(new CommandCreateDomain());
 
 		return cmds;
