@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -37,8 +37,6 @@ package net.sourceforge.plantuml.eggs;
 
 import java.awt.geom.Dimension2D;
 import java.awt.geom.Point2D;
-import java.io.IOException;
-import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -46,42 +44,39 @@ import java.util.Comparator;
 import java.util.Iterator;
 import java.util.List;
 
-import net.sourceforge.plantuml.AbstractPSystem;
 import net.sourceforge.plantuml.BackSlash;
 import net.sourceforge.plantuml.FileFormatOption;
+import net.sourceforge.plantuml.PlainDiagram;
 import net.sourceforge.plantuml.SpriteContainerEmpty;
 import net.sourceforge.plantuml.core.DiagramDescription;
-import net.sourceforge.plantuml.core.ImageData;
+import net.sourceforge.plantuml.core.UmlSource;
 import net.sourceforge.plantuml.cucadiagram.Display;
 import net.sourceforge.plantuml.graphic.FontConfiguration;
 import net.sourceforge.plantuml.graphic.HorizontalAlignment;
-import net.sourceforge.plantuml.graphic.HtmlColor;
-import net.sourceforge.plantuml.graphic.HtmlColorSetSimple;
-import net.sourceforge.plantuml.graphic.HtmlColorSimple;
-import net.sourceforge.plantuml.graphic.HtmlColorUtils;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.graphic.TextBlock;
 import net.sourceforge.plantuml.graphic.UDrawable;
-import net.sourceforge.plantuml.ugraphic.ColorMapperIdentity;
-import net.sourceforge.plantuml.ugraphic.ImageBuilder;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UFont;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UPolygon;
 import net.sourceforge.plantuml.ugraphic.URectangle;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
+import net.sourceforge.plantuml.ugraphic.color.HColorSet;
+import net.sourceforge.plantuml.ugraphic.color.HColorSimple;
+import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
 
 // http://www.redblobgames.com/grids/hexagons/
-public class PSystemColors extends AbstractPSystem implements UDrawable {
+public class PSystemColors extends PlainDiagram implements UDrawable {
 
 	private final double rectangleHeight = 28;
 	private final double rectangleWidth = 175;
-	private final HtmlColorSetSimple colors = new HtmlColorSetSimple();
+	private final HColorSet colors = HColorSet.instance();
 	private final String paletteCentralColor;
 	private final double size = 60;
 
-	public PSystemColors(String option) {
+	public PSystemColors(UmlSource source, String option) {
+		super(source);
 		if (option == null) {
 			this.paletteCentralColor = null;
 		} else {
@@ -90,12 +85,8 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 	}
 
 	@Override
-	final protected ImageData exportDiagramNow(OutputStream os, int num, FileFormatOption fileFormat, long seed)
-			throws IOException {
-		final ImageBuilder imageBuilder = new ImageBuilder(new ColorMapperIdentity(), 1.0, HtmlColorUtils.WHITE,
-				getMetadata(), null, 0, 0, null, false);
-		imageBuilder.setUDrawable(this);
-		return imageBuilder.writeImageTOBEMOVED(fileFormat, seed, os);
+	protected UDrawable getRootDrawable(FileFormatOption fileFormatOption) {
+		return this;
 	}
 
 	public DiagramDescription getDescription() {
@@ -103,7 +94,7 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 	}
 
 	public void drawU(UGraphic ug) {
-		if (colors.getColorIfValid(paletteCentralColor) instanceof HtmlColorSimple) {
+		if (paletteCentralColor != null && colors.getColorOrWhite(paletteCentralColor) instanceof HColorSimple) {
 			drawPalette(ug);
 		} else {
 			drawFull(ug);
@@ -154,7 +145,7 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 	}
 
 	private void drawOneHexa(UGraphic ug, String colorName, int i, int j, UPolygon hexa) {
-		final HtmlColorSimple color = (HtmlColorSimple) colors.getColorIfValid(colorName);
+		final HColorSimple color = (HColorSimple) colors.getColorOrWhite(colorName);
 		ug = applyColor(ug, color);
 		ug = ug.apply(new UTranslate(centerHexa(i, j)));
 		ug.draw(hexa);
@@ -178,7 +169,7 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 				continue;
 			}
 			final String candidat = colorName.substring(0, i) + BackSlash.BS_BS_N + colorName.substring(i);
-			final TextBlock tt = getTextName(font, candidat, (HtmlColorSimple) HtmlColorUtils.BLACK);
+			final TextBlock tt = getTextName(font, candidat, (HColorSimple) HColorUtils.BLACK);
 			final double width = tt.calculateDimension(stringBounder).getWidth();
 			if (width < min) {
 				result = candidat;
@@ -188,8 +179,8 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 		return result;
 	}
 
-	private UGraphic applyColor(UGraphic ug, HtmlColor color) {
-		return ug.apply(new UChangeColor(color)).apply(new UChangeBackColor(color));
+	private UGraphic applyColor(UGraphic ug, HColor color) {
+		return ug.apply(color).apply(color.bg());
 	}
 
 	private Point2D corner(int i) {
@@ -207,7 +198,7 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 	}
 
 	private List<String> getColorsCloseTo(String other) {
-		final List<String> result = new ArrayList<String>(colors.names());
+		final List<String> result = new ArrayList<>(colors.names());
 		for (Iterator<String> it = result.iterator(); it.hasNext();) {
 			final String candidat = it.next();
 			final String similar = candidat.replaceAll("Gray", "Grey");
@@ -235,11 +226,11 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 	}
 
 	private Comparator<String> closeComparator(String center) {
-		final HtmlColorSimple centerColor = (HtmlColorSimple) colors.getColorIfValid(center);
+		final HColorSimple centerColor = (HColorSimple) colors.getColorOrWhite(center);
 		return new Comparator<String>() {
 			public int compare(String col1, String col2) {
-				final double dist1 = centerColor.distance((HtmlColorSimple) colors.getColorIfValid(col1));
-				final double dist2 = centerColor.distance((HtmlColorSimple) colors.getColorIfValid(col2));
+				final double dist1 = centerColor.distance((HColorSimple) colors.getColorOrWhite(col1));
+				final double dist2 = centerColor.distance((HColorSimple) colors.getColorOrWhite(col2));
 				return (int) Math.signum(dist1 - dist2);
 			}
 		};
@@ -248,12 +239,12 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 	private void drawFull(UGraphic ug) {
 		final UFont font = UFont.sansSerif(14).bold();
 
-		ug = ug.apply(new UChangeColor(HtmlColorUtils.BLACK));
+		ug = ug.apply(HColorUtils.BLACK);
 		int i = 0;
 		int j = 0;
 		for (String name : colors.names()) {
 			UGraphic tmp = getPositioned(ug, i, j);
-			final HtmlColorSimple color = (HtmlColorSimple) colors.getColorIfValid(name);
+			final HColorSimple color = (HColorSimple) colors.getColorOrWhite(name);
 			applyColor(tmp, color).draw(new URectangle(rectangleWidth, rectangleHeight));
 			final TextBlock tt = getTextName(font, name, color);
 			final Dimension2D dimText = tt.calculateDimension(ug.getStringBounder());
@@ -267,9 +258,9 @@ public class PSystemColors extends AbstractPSystem implements UDrawable {
 		}
 	}
 
-	private TextBlock getTextName(final UFont font, String name, final HtmlColorSimple color) {
-		final HtmlColorSimple opposite = color.opposite();
-		final FontConfiguration fc = new FontConfiguration(font, opposite, HtmlColorUtils.BLUE, true);
+	private TextBlock getTextName(final UFont font, String name, final HColorSimple color) {
+		final HColorSimple opposite = color.opposite();
+		final FontConfiguration fc = new FontConfiguration(font, opposite, HColorUtils.BLUE, true);
 		final TextBlock tt = Display.getWithNewlines(name).create(fc, HorizontalAlignment.CENTER,
 				new SpriteContainerEmpty());
 		return tt;
