@@ -48,6 +48,7 @@ import java.io.PrintWriter;
 import java.net.InetAddress;
 import java.net.ServerSocket;
 import java.net.Socket;
+import java.net.URLEncoder;
 import java.util.Collections;
 import java.util.Date;
 import java.util.List;
@@ -175,10 +176,10 @@ public class PicoWebServer implements Runnable {
 		} catch (Exception e) {
 			throw new BadRequest400("Error parsing request json: " + e.getMessage(), e);
 		}
-		
+
 		handleRenderRequest(renderRequest, out);
 	}
-	
+
 	public void handleRenderRequest(RenderRequest renderRequest, BufferedOutputStream out) throws Exception {
 
 		final Option option = new Option(renderRequest.getOptions());
@@ -226,6 +227,12 @@ public class PicoWebServer implements Runnable {
 				write(out, "X-PlantUML-Diagram-Error-Line: " + (1 + err.getLineLocation().getPosition()));
 			}
 		}
+		if (system.getTitleDisplay() != null && system.getTitleDisplay().size() == 1) {
+			final String encode = URLEncoder.encode(system.getTitleDisplay().asList().get(0).toString(), "UTF-8");
+			if (encode.length() < 256)
+				write(out, "X-PlantUML-Diagram-Title: " + encode);
+		}
+
 		write(out, "X-Patreon: Support us on https://plantuml.com/patreon");
 		write(out, "X-Donate: https://plantuml.com/paypal");
 		write(out, "X-Quote: " + StringUtils.rot(QuoteUtils.getSomeQuote()));
