@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -38,7 +38,7 @@ package net.sourceforge.plantuml.posimo;
 import java.awt.geom.CubicCurve2D;
 import java.awt.geom.Point2D;
 import java.util.Collection;
-import java.util.Map;
+import java.util.Objects;
 
 import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.ISkinParam;
@@ -46,8 +46,6 @@ import net.sourceforge.plantuml.cucadiagram.LinkDecor;
 import net.sourceforge.plantuml.cucadiagram.LinkStyle;
 import net.sourceforge.plantuml.cucadiagram.LinkType;
 import net.sourceforge.plantuml.skin.rose.Rose;
-import net.sourceforge.plantuml.ugraphic.UChangeBackColor;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.UPolygon;
 import net.sourceforge.plantuml.ugraphic.URectangle;
@@ -134,14 +132,14 @@ public class PathDrawerInterface implements PathDrawer {
 //			final Map<Point2D, Double> all = dotPath.somePoints();
 //			final Point2D p = getFarest(outPoint, inPoint, all.keySet());
 //
-//			ug = ug.apply(new UChangeBackColor(rose.getHtmlColor(param, ColorParam.background)));
-//			ug = ug.apply(new UChangeColor(rose.getHtmlColor(param, ColorParam.classBorder)));
+//			ug = ug.apply(UChangeBackColor.nnn(rose.getHtmlColor(param, ColorParam.background)));
+//			ug = ug.apply(UChangeColor.nnn(rose.getHtmlColor(param, ColorParam.classBorder)));
 //
 //			decor.drawDecor(ug, p, all.get(p));
 //		}
 
 		throw new UnsupportedOperationException();
-//		ug = ug.apply(new UChangeColor(rose.getHtmlColor(param, ColorParam.classBorder)));
+//		ug = ug.apply(UChangeColor.nnn(rose.getHtmlColor(param, ColorParam.classBorder)));
 //		if (linkType.isDashed()) {
 //			goDash(ug);
 //		}
@@ -167,7 +165,7 @@ public class PathDrawerInterface implements PathDrawer {
 		// // final double theta = Math.atan2(
 		// // -direction.getX() + position.getX(), direction.getY()
 		// // - position.getY());
-		// if (decor == LinkDecor.SQUARRE) {
+		// if (decor == LinkDecor.SQUARE) {
 		// middle1 = drawSquare(ug, position.getX(), position.getY());
 		// } else if (decor == LinkDecor.EXTENDS) {
 		// middle1 = drawExtends(ug, position.getX(), position.getY(), theta);
@@ -210,15 +208,12 @@ public class PathDrawerInterface implements PathDrawer {
 				farest = candidat;
 			}
 		}
-		if (result == null) {
-			throw new IllegalArgumentException();
-		}
-		return result;
+		return Objects.requireNonNull(result);
 	}
 
 	private Point2D drawSquare(UGraphic ug, double centerX, double centerY) {
-		ug = ug.apply(new UChangeBackColor(rose.getHtmlColor(param, ColorParam.classBackground)));
-		ug = ug.apply(new UChangeColor(rose.getHtmlColor(param, ColorParam.classBorder)));
+		ug = ug.apply(rose.getHtmlColor(param, ColorParam.classBackground).bg());
+		ug = ug.apply(rose.getHtmlColor(param, ColorParam.classBorder));
 		final double width = 10;
 		final double height = 10;
 		ug.apply(new UTranslate(centerX - width / 2, centerY - height / 2)).draw(new URectangle(width, height));
@@ -226,8 +221,8 @@ public class PathDrawerInterface implements PathDrawer {
 	}
 
 	Point2D drawExtends(UGraphic ug, double x, double y, double theta) {
-		ug = ug.apply(new UChangeBackColor(rose.getHtmlColor(param, ColorParam.background)));
-		ug = ug.apply(new UChangeColor(rose.getHtmlColor(param, ColorParam.classBorder)));
+		ug = ug.apply(rose.getHtmlColor(param, ColorParam.background).bg());
+		ug = ug.apply(rose.getHtmlColor(param, ColorParam.classBorder));
 
 		// final double theta = Math.atan2(-pathPoint.getX() + x,
 		// pathPoint.getY() - y);
