@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,25 +35,36 @@
  */
 package net.sourceforge.plantuml.preproc;
 
-import java.io.File;
 import java.text.SimpleDateFormat;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Date;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Objects;
+import java.util.Set;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
+import net.sourceforge.plantuml.AParentFolder;
 import net.sourceforge.plantuml.Log;
 import net.sourceforge.plantuml.api.ApiWarning;
+import net.sourceforge.plantuml.security.SFile;
+import net.sourceforge.plantuml.security.SecurityProfile;
+import net.sourceforge.plantuml.security.SecurityUtils;
+import net.sourceforge.plantuml.tim.EaterException;
+import net.sourceforge.plantuml.tim.TMemory;
+import net.sourceforge.plantuml.tim.TVariableScope;
 import net.sourceforge.plantuml.version.Version;
 
 public class Defines implements Truth {
 
 	private final Map<String, String> environment = new LinkedHashMap<String, String>();
 	private final Map<String, Define> values = new LinkedHashMap<String, Define>();
-	private final Map<String, Define> savedState = new LinkedHashMap<String, Define>();
 
 	@Deprecated
 	@ApiWarning(willBeRemoved = "in next major release")
@@ -63,13 +74,22 @@ public class Defines implements Truth {
 
 	@Override
 	public String toString() {
-		return values.keySet().toString();
+		return values.keySet().toString() + " " + environment.keySet();
 	}
 
 	public static Defines createEmpty() {
 		return new Defines();
 	}
 
+	public void copyTo(TMemory memory) throws EaterException {
+		for (Entry<String, Define> ent : values.entrySet()) {
+			final String name = ent.getKey();
+			final Define def = ent.getValue();
+			memory.putVariable(name, def.asTVariable(), TVariableScope.GLOBAL);
+		}
+
+	}
+
 	public void overrideFilename(String filename) {
 		if (filename != null) {
 			environment.put("filename", filename);
@@ -80,6 +100,7 @@ public class Defines implements Truth {
 	public void importFrom(Defines other) {
 		this.environment.putAll(other.environment);
 		this.values.putAll(other.values);
+		magic = null;
 	}
 
 	public Defines cloneMe() {
@@ -88,19 +109,42 @@ public class Defines implements Truth {
 		return result;
 	}
 
-	public static Defines createWithFileName(File file) {
-		if (file == null) {
-			throw new IllegalArgumentException();
-		}
+	public static Defines createWithFileName(SFile file) {
+		Objects.requireNonNull(file);
 		final Defines result = createEmpty();
 		result.overrideFilename(file.getName());
 		result.environment.put("filedate", new Date(file.lastModified()).toString());
-		// result.environment.put("filename", file.getName());
-		// result.environment.put("filenameNoExtension", nameNoExtension(file));
-		result.environment.put("dirpath", file.getAbsoluteFile().getParentFile().getAbsolutePath().replace('\\', '/'));
+		if (SecurityUtils.getSecurityProfile() == SecurityProfile.UNSECURE) {
+			result.environment.put("dirpath",
+					file.getAbsoluteFile().getParentFile().getAbsolutePath().replace('\\', '/'));
+		}
 		return result;
 	}
 
+	public static Defines createWithFileName(java.io.File file) {
+		Objects.requireNonNull(file);
+		final Defines result = createEmpty();
+		result.overrideFilename(file.getName());
+		result.environment.put("filedate", new Date(file.lastModified()).toString());
+		if (SecurityUtils.getSecurityProfile() == SecurityProfile.UNSECURE) {
+			result.environment.put("dirpath",
+					file.getAbsoluteFile().getParentFile().getAbsolutePath().replace('\\', '/'));
+		}
+		return result;
+	}
+
+	private static Defines createWithMap(Map<String, String> init) {
+		final Defines result = createEmpty();
+		for (Map.Entry<String, String> ent : init.entrySet()) {
+			result.environment.put(ent.getKey(), ent.getValue());
+		}
+		return result;
+	}
+
+	public String getEnvironmentValue(String key) {
+		return this.environment.get(key);
+	}
+
 	private static String nameNoExtension(String name) {
 		final int x = name.lastIndexOf('.');
 		if (x == -1) {
@@ -109,8 +153,9 @@ public class Defines implements Truth {
 		return name.substring(0, x);
 	}
 
-	public void define(String name, List<String> value, boolean emptyParentheses) {
-		values.put(name, new Define(name, value, emptyParentheses));
+	public void define(String name, List<String> value, boolean emptyParentheses, AParentFolder currentDir) {
+		values.put(name, new Define(name, value, emptyParentheses, currentDir));
+		magic = null;
 	}
 
 	public boolean isDefine(String expression) {
@@ -134,16 +179,67 @@ public class Defines implements Truth {
 
 	public void undefine(String name) {
 		values.remove(name);
+		magic = null;
 	}
 
 	public List<String> applyDefines(String line) {
+		// System.err.println("line=" + line + " " + values.size());
 		line = manageDate(line);
 		line = manageEnvironment(line);
-		for (Map.Entry<String, Define> ent : values.entrySet()) {
-			final Define def = ent.getValue();
+		line = method1(line);
+		// line = values.size() < 10 ? method1(line) : method2(line);
+		return Arrays.asList(line.split("\n"));
+	}
+
+	private String method1(String line) {
+		for (Define def : values.values()) {
 			line = def.apply(line);
 		}
-		return Arrays.asList(line.split("\n"));
+		return line;
+	}
+
+	private Map<String, Collection<Define>> getAll() {
+		final Map<String, Collection<Define>> result = new LinkedHashMap<String, Collection<Define>>();
+		for (Define def : values.values()) {
+			Collection<Define> tmp = result.get(def.getFunctionName());
+			if (tmp == null) {
+				tmp = new ArrayList<>();
+				result.put(def.getFunctionName(), tmp);
+			}
+			tmp.add(def);
+		}
+		return result;
+	}
+
+	private Map<String, Collection<Define>> magic;
+
+	private String method2(String line) {
+		final Set<String> words = words(line);
+		if (magic == null) {
+			magic = getAll();
+
+		}
+		for (String w : words) {
+			Collection<Define> tmp = magic.get(w);
+			if (tmp == null) {
+				continue;
+			}
+			for (Define def : tmp) {
+				line = def.apply(line);
+			}
+		}
+		return line;
+	}
+
+	private Set<String> words(String line) {
+		final String ID = "[A-Za-z_][A-Za-z_0-9]*";
+		Pattern p = Pattern.compile(ID);
+		Matcher m = p.matcher(line);
+		final Set<String> words = new HashSet<>();
+		while (m.find()) {
+			words.add(m.group(0));
+		}
+		return words;
 	}
 
 	private String manageEnvironment(String line) {
@@ -176,18 +272,4 @@ public class Defines implements Truth {
 		return line;
 	}
 
-	public void saveState() {
-		if (savedState.size() > 0) {
-			throw new IllegalStateException();
-		}
-		this.savedState.putAll(values);
-
-	}
-
-	public void restoreState() {
-		this.values.clear();
-		this.values.putAll(savedState);
-
-	}
-
 }
