@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -35,16 +35,15 @@
  */
 package net.sourceforge.plantuml.preproc;
 
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileReader;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
-import java.net.URL;
+import java.io.Reader;
+import java.nio.charset.Charset;
 
-import net.sourceforge.plantuml.CharSequence2;
 import net.sourceforge.plantuml.Log;
+import net.sourceforge.plantuml.StringLocated;
+import net.sourceforge.plantuml.security.SURL;
 import net.sourceforge.plantuml.utils.StartUtils;
 
 public class StartDiagramExtractReader implements ReadLine {
@@ -52,16 +51,16 @@ public class StartDiagramExtractReader implements ReadLine {
 	private final ReadLine raw;
 	private boolean finished = false;
 
-	public StartDiagramExtractReader(CharSequence2 s, File f, String uid, String charset) {
-		this(getReadLine(s, f, charset), uid);
+	public static StartDiagramExtractReader build(FileWithSuffix f2, StringLocated s, Charset charset) {
+		return new StartDiagramExtractReader(getReadLine(f2, s, charset), f2.getSuffix());
 	}
 
-	public StartDiagramExtractReader(CharSequence2 s, URL url, String uid, String charset) {
-		this(getReadLine(s, url, charset), uid);
+	public static StartDiagramExtractReader build(SURL url, StringLocated s, String uid, Charset charset) {
+		return new StartDiagramExtractReader(getReadLine(url, s, charset), uid);
 	}
 
-	public StartDiagramExtractReader(CharSequence2 s, InputStream is, String desc) {
-		this(getReadLine(s, is, desc), null);
+	public static StartDiagramExtractReader build(InputStream is, StringLocated s, String desc) {
+		return new StartDiagramExtractReader(getReadLine(is, s, desc), null);
 	}
 
 	private StartDiagramExtractReader(ReadLine raw, String suf) {
@@ -76,10 +75,10 @@ public class StartDiagramExtractReader implements ReadLine {
 			bloc = 0;
 		}
 		this.raw = raw;
-		CharSequence2 s = null;
+		StringLocated s = null;
 		try {
 			while ((s = raw.readLine()) != null) {
-				if (StartUtils.isArobaseStartDiagram(s) && checkUid(uid, s)) {
+				if (StartUtils.isArobaseStartDiagram(s.getString()) && checkUid(uid, s)) {
 					if (bloc == 0) {
 						return;
 					}
@@ -93,7 +92,7 @@ public class StartDiagramExtractReader implements ReadLine {
 		finished = true;
 	}
 
-	private boolean checkUid(String uid, CharSequence2 s) {
+	private boolean checkUid(String uid, StringLocated s) {
 		if (uid == null) {
 			return true;
 		}
@@ -103,59 +102,50 @@ public class StartDiagramExtractReader implements ReadLine {
 		return false;
 	}
 
-	private static ReadLine getReadLine(CharSequence2 s, File f, String charset) {
+	private static ReadLine getReadLine(FileWithSuffix f2, StringLocated s, Charset charset) {
 		try {
-			if (charset == null) {
-				Log.info("Using default charset");
-				return new UncommentReadLine(ReadLineReader.create(new FileReader(f), f.getAbsolutePath()));
+			final Reader tmp1 = f2.getReader(charset);
+			if (tmp1 == null) {
+				return new ReadLineSimple(s, "Cannot open " + f2.getDescription());
 			}
-			Log.info("Using charset " + charset);
-			return new UncommentReadLine(ReadLineReader.create(new InputStreamReader(new FileInputStream(f), charset),
-					f.getAbsolutePath()));
+			return new UncommentReadLine(ReadLineReader.create(tmp1, f2.getDescription()));
 		} catch (IOException e) {
 			return new ReadLineSimple(s, e.toString());
 		}
 	}
 
-	private static ReadLine getReadLine(CharSequence2 s, InputStream is, String description) {
+	private static ReadLine getReadLine(InputStream is, StringLocated s, String description) {
 		return new UncommentReadLine(ReadLineReader.create(new InputStreamReader(is), description));
 	}
 
-	private static ReadLine getReadLine(CharSequence2 s, URL url, String charset) {
-		try {
-			if (charset == null) {
-				Log.info("Using default charset");
-				return new UncommentReadLine(ReadLineReader.create(new InputStreamReader(url.openStream()),
-						url.toString()));
-			}
-			Log.info("Using charset " + charset);
-			return new UncommentReadLine(ReadLineReader.create(new InputStreamReader(url.openStream(), charset),
-					url.toString()));
-		} catch (IOException e) {
-			return new ReadLineSimple(s, e.toString());
+	private static ReadLine getReadLine(SURL url, StringLocated s, Charset charset) {
+		final InputStream tmp = url.openStream();
+		if (tmp == null) {
+			return new ReadLineSimple(s, "Cannot connect");
 		}
+		return new UncommentReadLine(ReadLineReader.create(new InputStreamReader(tmp, charset), url.toString()));
 	}
 
-	static public boolean containsStartDiagram(CharSequence2 s, File f, String charset) throws IOException {
-		final ReadLine r = getReadLine(s, f, charset);
+	static public boolean containsStartDiagram(FileWithSuffix f2, StringLocated s, Charset charset) throws IOException {
+		final ReadLine r = getReadLine(f2, s, charset);
 		return containsStartDiagram(r);
 	}
 
-	static public boolean containsStartDiagram(CharSequence2 s, URL url, String charset) throws IOException {
-		final ReadLine r = getReadLine(s, url, charset);
+	static public boolean containsStartDiagram(SURL url, StringLocated s, Charset charset) throws IOException {
+		final ReadLine r = getReadLine(url, s, charset);
 		return containsStartDiagram(r);
 	}
 
-	static public boolean containsStartDiagram(CharSequence2 s, InputStream is, String description) throws IOException {
-		final ReadLine r = getReadLine(s, is, description);
+	static public boolean containsStartDiagram(InputStream is, StringLocated s, String description) throws IOException {
+		final ReadLine r = getReadLine(is, s, description);
 		return containsStartDiagram(r);
 	}
 
 	private static boolean containsStartDiagram(final ReadLine r) throws IOException {
 		try {
-			CharSequence2 s = null;
+			StringLocated s = null;
 			while ((s = r.readLine()) != null) {
-				if (StartUtils.isArobaseStartDiagram(s)) {
+				if (StartUtils.isArobaseStartDiagram(s.getString())) {
 					return true;
 				}
 			}
@@ -167,12 +157,12 @@ public class StartDiagramExtractReader implements ReadLine {
 		return false;
 	}
 
-	public CharSequence2 readLine() throws IOException {
+	public StringLocated readLine() throws IOException {
 		if (finished) {
 			return null;
 		}
-		final CharSequence2 result = raw.readLine();
-		if (result != null && StartUtils.isArobaseEndDiagram(result)) {
+		final StringLocated result = raw.readLine();
+		if (result != null && StartUtils.isArobaseEndDiagram(result.getString())) {
 			finished = true;
 			return null;
 		}
