@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -38,21 +38,26 @@ package net.sourceforge.plantuml.sequencediagram;
 import java.util.ArrayList;
 import java.util.List;
 
+import net.sourceforge.plantuml.ISkinSimple;
 import net.sourceforge.plantuml.command.Command;
-import net.sourceforge.plantuml.command.UmlDiagramFactory;
+import net.sourceforge.plantuml.command.PSystemCommandFactory;
+import net.sourceforge.plantuml.command.note.sequence.FactorySequenceNoteAcrossCommand;
 import net.sourceforge.plantuml.command.note.sequence.FactorySequenceNoteCommand;
 import net.sourceforge.plantuml.command.note.sequence.FactorySequenceNoteOnArrowCommand;
 import net.sourceforge.plantuml.command.note.sequence.FactorySequenceNoteOverSeveralCommand;
+import net.sourceforge.plantuml.core.UmlSource;
 import net.sourceforge.plantuml.sequencediagram.command.CommandActivate;
 import net.sourceforge.plantuml.sequencediagram.command.CommandActivate2;
 import net.sourceforge.plantuml.sequencediagram.command.CommandArrow;
 import net.sourceforge.plantuml.sequencediagram.command.CommandAutoNewpage;
 import net.sourceforge.plantuml.sequencediagram.command.CommandAutoactivate;
 import net.sourceforge.plantuml.sequencediagram.command.CommandAutonumber;
+import net.sourceforge.plantuml.sequencediagram.command.CommandAutonumberIncrement;
 import net.sourceforge.plantuml.sequencediagram.command.CommandAutonumberResume;
 import net.sourceforge.plantuml.sequencediagram.command.CommandAutonumberStop;
 import net.sourceforge.plantuml.sequencediagram.command.CommandBoxEnd;
 import net.sourceforge.plantuml.sequencediagram.command.CommandBoxStart;
+import net.sourceforge.plantuml.sequencediagram.command.CommandDeactivateShort;
 import net.sourceforge.plantuml.sequencediagram.command.CommandDelay;
 import net.sourceforge.plantuml.sequencediagram.command.CommandDivider;
 import net.sourceforge.plantuml.sequencediagram.command.CommandExoArrowLeft;
@@ -61,7 +66,9 @@ import net.sourceforge.plantuml.sequencediagram.command.CommandFootbox;
 import net.sourceforge.plantuml.sequencediagram.command.CommandFootboxOld;
 import net.sourceforge.plantuml.sequencediagram.command.CommandGrouping;
 import net.sourceforge.plantuml.sequencediagram.command.CommandHSpace;
+import net.sourceforge.plantuml.sequencediagram.command.CommandHideUnlinked;
 import net.sourceforge.plantuml.sequencediagram.command.CommandIgnoreNewpage;
+import net.sourceforge.plantuml.sequencediagram.command.CommandLinkAnchor;
 import net.sourceforge.plantuml.sequencediagram.command.CommandNewpage;
 import net.sourceforge.plantuml.sequencediagram.command.CommandParticipantA;
 import net.sourceforge.plantuml.sequencediagram.command.CommandParticipantA2;
@@ -70,24 +77,25 @@ import net.sourceforge.plantuml.sequencediagram.command.CommandParticipantA4;
 import net.sourceforge.plantuml.sequencediagram.command.CommandReferenceMultilinesOverSeveral;
 import net.sourceforge.plantuml.sequencediagram.command.CommandReferenceOverSeveral;
 import net.sourceforge.plantuml.sequencediagram.command.CommandReturn;
-import net.sourceforge.plantuml.sequencediagram.command.CommandSkin;
 import net.sourceforge.plantuml.sequencediagram.command.CommandUrl;
 
-public class SequenceDiagramFactory extends UmlDiagramFactory {
+public class SequenceDiagramFactory extends PSystemCommandFactory {
 
 	@Override
-	public SequenceDiagram createEmptyDiagram() {
-		return new SequenceDiagram();
+	public SequenceDiagram createEmptyDiagram(UmlSource source, ISkinSimple skinParam) {
+		return new SequenceDiagram(source, skinParam);
 	}
 
 	@Override
 	protected List<Command> createCommands() {
 
-		final List<Command> cmds = new ArrayList<Command>();
+		final List<Command> cmds = new ArrayList<>();
 
-		addCommonCommands(cmds);
+		addCommonCommands1(cmds);
+		cmds.add(new CommandHideUnlinked());
 
 		cmds.add(new CommandActivate());
+		cmds.add(new CommandDeactivateShort());
 
 		cmds.add(new CommandParticipantA());
 		cmds.add(new CommandParticipantA2());
@@ -103,6 +111,8 @@ public class SequenceDiagramFactory extends UmlDiagramFactory {
 
 		final FactorySequenceNoteOverSeveralCommand factorySequenceNoteOverSeveralCommand = new FactorySequenceNoteOverSeveralCommand();
 		cmds.add(factorySequenceNoteOverSeveralCommand.createSingleLine());
+		final FactorySequenceNoteAcrossCommand factorySequenceNoteAcrossCommand = new FactorySequenceNoteAcrossCommand();
+		cmds.add(factorySequenceNoteAcrossCommand.createSingleLine());
 
 		cmds.add(new CommandBoxStart());
 		cmds.add(new CommandBoxEnd());
@@ -117,6 +127,7 @@ public class SequenceDiagramFactory extends UmlDiagramFactory {
 		cmds.add(factorySequenceNoteCommand.createMultiLine(false));
 		cmds.add(factorySequenceNoteOverSeveralCommand.createMultiLine(false));
 		cmds.add(factorySequenceNoteOnArrowCommand.createMultiLine(false));
+		cmds.add(factorySequenceNoteAcrossCommand.createMultiLine(false));
 
 		cmds.add(new CommandNewpage());
 		cmds.add(new CommandIgnoreNewpage());
@@ -125,15 +136,16 @@ public class SequenceDiagramFactory extends UmlDiagramFactory {
 		cmds.add(new CommandHSpace());
 		cmds.add(new CommandReferenceOverSeveral());
 		cmds.add(new CommandReferenceMultilinesOverSeveral());
-		cmds.add(new CommandSkin());
 		cmds.add(new CommandAutonumber());
 		cmds.add(new CommandAutonumberStop());
 		cmds.add(new CommandAutonumberResume());
+		cmds.add(new CommandAutonumberIncrement());
 		cmds.add(new CommandAutoactivate());
 		cmds.add(new CommandFootbox());
 		cmds.add(new CommandDelay());
 		cmds.add(new CommandFootboxOld());
 		cmds.add(new CommandUrl());
+		cmds.add(new CommandLinkAnchor());
 
 		return cmds;
 	}
