@@ -2,7 +2,7 @@
  * PlantUML : a free UML diagram generator
  * ========================================================================
  *
- * (C) Copyright 2009-2017, Arnaud Roques
+ * (C) Copyright 2009-2020, Arnaud Roques
  *
  * Project Info:  http://plantuml.com
  * 
@@ -41,21 +41,22 @@ import java.util.List;
 import net.sourceforge.plantuml.ColorParam;
 import net.sourceforge.plantuml.Dimension2DDouble;
 import net.sourceforge.plantuml.ISkinParam;
+import net.sourceforge.plantuml.cucadiagram.Stereotype;
 import net.sourceforge.plantuml.graphic.AbstractTextBlock;
-import net.sourceforge.plantuml.graphic.HtmlColor;
 import net.sourceforge.plantuml.graphic.StringBounder;
 import net.sourceforge.plantuml.skin.rose.Rose;
-import net.sourceforge.plantuml.ugraphic.UChangeColor;
 import net.sourceforge.plantuml.ugraphic.UGraphic;
 import net.sourceforge.plantuml.ugraphic.ULine;
 import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
+import net.sourceforge.plantuml.ugraphic.color.HColor;
 
 public final class CucaDiagramFileMakerSvek2InternalImage extends AbstractTextBlock implements IEntityImage {
 
 	private final List<IEntityImage> inners;
 	private final Separator separator;
 	private final ISkinParam skinParam;
+	private final Stereotype stereotype;
 
 	static enum Separator {
 		VERTICAL, HORIZONTAL;
@@ -72,9 +73,9 @@ public final class CucaDiagramFileMakerSvek2InternalImage extends AbstractTextBl
 
 		UTranslate move(Dimension2D dim) {
 			if (this == VERTICAL) {
-				return new UTranslate(dim.getWidth(), 0);
+				return UTranslate.dx(dim.getWidth());
 			}
-			return new UTranslate(0, dim.getHeight());
+			return UTranslate.dy(dim.getHeight());
 		}
 
 		Dimension2D add(Dimension2D orig, Dimension2D other) {
@@ -91,27 +92,28 @@ public final class CucaDiagramFileMakerSvek2InternalImage extends AbstractTextBl
 			final int DASH = 8;
 			ug = ug.apply(new UStroke(DASH, 10, THICKNESS_BORDER));
 			if (this == VERTICAL) {
-				ug.draw(new ULine(0, dimTotal.getHeight() + DASH));
+				ug.draw(ULine.vline(dimTotal.getHeight() + DASH));
 			} else {
-				ug.draw(new ULine(dimTotal.getWidth() + DASH, 0));
+				ug.draw(ULine.hline(dimTotal.getWidth() + DASH));
 			}
 
 		}
 	}
 
-	private HtmlColor getColor(ColorParam colorParam) {
-		return new Rose().getHtmlColor(skinParam, colorParam);
+	private HColor getColor(ColorParam colorParam, Stereotype stereotype) {
+		return new Rose().getHtmlColor(skinParam, stereotype, colorParam);
 	}
 
 	public CucaDiagramFileMakerSvek2InternalImage(List<IEntityImage> inners, char concurrentSeparator,
-			ISkinParam skinParam) {
+			ISkinParam skinParam, Stereotype stereotype) {
 		this.separator = Separator.fromChar(concurrentSeparator);
 		this.skinParam = skinParam;
+		this.stereotype = stereotype;
 		this.inners = inners;
 	}
 
 	public void drawU(UGraphic ug) {
-		final HtmlColor dotColor = getColor(ColorParam.stateBorder);
+		final HColor dotColor = getColor(ColorParam.stateBorder, stereotype);
 		final StringBounder stringBounder = ug.getStringBounder();
 		final Dimension2D dimTotal = calculateDimension(stringBounder);
 
@@ -121,7 +123,7 @@ public final class CucaDiagramFileMakerSvek2InternalImage extends AbstractTextBl
 			final Dimension2D dim = inner.calculateDimension(stringBounder);
 			ug = ug.apply(separator.move(dim));
 			if (i < inners.size() - 1) {
-				separator.drawSeparator(ug.apply(new UChangeColor(dotColor)), dimTotal);
+				separator.drawSeparator(ug.apply(dotColor), dimTotal);
 			}
 		}
 
@@ -136,9 +138,13 @@ public final class CucaDiagramFileMakerSvek2InternalImage extends AbstractTextBl
 		return result;
 	}
 
-	public HtmlColor getBackcolor() {
+	public HColor getBackcolor() {
 		return skinParam.getBackgroundColor();
 	}
+	
+	public double getOverscanX(StringBounder stringBounder) {
+		return 0;
+	}
 
 	public boolean isHidden() {
 		return false;
