@@ -101,7 +101,6 @@ import net.sourceforge.plantuml.ugraphic.UStroke;
 import net.sourceforge.plantuml.ugraphic.UTranslate;
 import net.sourceforge.plantuml.ugraphic.color.HColor;
 import net.sourceforge.plantuml.ugraphic.color.HColorNone;
-import net.sourceforge.plantuml.ugraphic.color.HColorUtils;
 
 public class SvekLine implements Moveable, Hideable, GuideLine {
 
@@ -230,11 +229,11 @@ public class SvekLine implements Moveable, Hideable, GuideLine {
 			skinParam = link.getColors().mute(skinParam);
 			font = font.mute(link.getColors());
 		}
-		this.backgroundColor = skinParam.getBackgroundColor(false);
+		this.backgroundColor = skinParam.getBackgroundColor();
 		this.defaultThickness = skinParam.getThickness(LineParam.arrow, null);
 		this.arrowLollipopColor = skinParam.getHtmlColor(ColorParam.arrowLollipop, null, false);
 		if (arrowLollipopColor == null) {
-			this.arrowLollipopColor = HColorUtils.WHITE;
+			this.arrowLollipopColor = backgroundColor;
 		}
 		this.pragma = pragma;
 		this.bibliotekon = bibliotekon;
@@ -1042,4 +1041,4 @@ public class SvekLine implements Moveable, Hideable, GuideLine {
 		return link.getStereotype();
 	}
 
-}
\ No newline at end of file
+}
