@@ -100,7 +100,7 @@ public abstract class EntityImageStateCommon extends AbstractEntityImage {
 	}
 
 	final protected UStroke getStroke() {
-		UStroke stroke = lineConfig.getColors(getSkinParam()).getSpecificLineStroke();
+		UStroke stroke = lineConfig.getColors().getSpecificLineStroke();
 		if (stroke == null) {
 			stroke = new UStroke(1.5);
 		}
@@ -130,16 +130,16 @@ public abstract class EntityImageStateCommon extends AbstractEntityImage {
 
 	final protected UGraphic applyColor(UGraphic ug) {
 
-		HColor classBorder = lineConfig.getColors(getSkinParam()).getColor(ColorType.LINE);
-		if (classBorder == null) {
+		HColor border = lineConfig.getColors().getColor(ColorType.LINE);
+		if (border == null) {
 			if (UseStyle.useBetaStyle())
-				classBorder = getStyleState().value(PName.LineColor).asColor(getSkinParam().getThemeStyle(),
+				border = getStyleState().value(PName.LineColor).asColor(getSkinParam().getThemeStyle(),
 						getSkinParam().getIHtmlColorSet());
 			else
-				classBorder = SkinParamUtils.getColor(getSkinParam(), getStereo(), ColorParam.stateBorder);
+				border = SkinParamUtils.getColor(getSkinParam(), getStereo(), ColorParam.stateBorder);
 		}
-		ug = ug.apply(getStroke()).apply(classBorder);
-		HColor backcolor = getEntity().getColors(getSkinParam()).getColor(ColorType.BACK);
+		ug = ug.apply(getStroke()).apply(border);
+		HColor backcolor = getEntity().getColors().getColor(ColorType.BACK);
 		if (backcolor == null) {
 			if (UseStyle.useBetaStyle())
 				backcolor = getStyleState().value(PName.BackGroundColor).asColor(getSkinParam().getThemeStyle(),
